﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using DevExpress.ExpressApp.DC;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.Utils;
using DevExpress.Persistent.Base;
using DevExpress.Xpo;
using Ticket36921.Module.Annotations;
using Ticket36921.Module.BusinessObjects;
using Xafari;
using Xafari.BC;
using Xafari.BC.BusinessOperations;
using Xafari.BC.BusinessOperations.Attributes;
using Xafari.BC.Model;
using Xafari.BC.Settings;
using Xafari.Model;
using Xafari.Utils;

namespace Ticket36921.Module.BusinessOperations
{
	[System.ComponentModel.DisplayName(@"RuntimeCalcOrderTotal")]
	[DefaultOperationService(typeof(RuntimeCalc))]
	[ExecutionWay(ExecutionWays.Synchronous)]
	[DomainComponent]
	public class RuntimeBusinessOperation : BusinessOperationBase, INotifyPropertyChanged
	{
		// Context property is marked with special attribute. Analyzing the type of property, 
		// framework define context of business operation and initializes a property before it is executed.
		[ContextProperty]
		[VisibleInDetailView(false)]
		public Order Order { get; set; }

		public string OrderNum { get { return Order.Number; } }

		private string _expression;

		[EditorAlias(XafariEditorAliases.ExpressionPropertyEditor)]
		[CriteriaOptions("Type")]
		public string Expression
		{
			get { return _expression;}
			set
			{
				_expression = value;
				this.RaiseChanged();
			}
		}

		[System.ComponentModel.DisplayName(@"Total cost of the order")]
		[VisibleInDetailView(false)]
		public decimal Total { get; internal set; }

		[ImmediatePostData]
		[ValueConverter(typeof(TypeToStringConverter))]
		[VisibleInDetailView(false)]
		public Type Type
		{
			get
			{
				return typeof(Order);
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;

		[NotifyPropertyChangedInvocator]
		protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
		{
			PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
		}

		public void RaiseChanged()
		{
			this.OnPropertyChanged(nameof(Expression));
		}
	}

	[System.ComponentModel.DisplayName(@"RuntimeCalcOrderTotal discounts"), Description("Calculates the total cost of  the list orders without discounts on items.")]
	[BusinessOperation(typeof(RuntimeBusinessOperation))]
	public class RuntimeCalc : OperationServiceBase
	{
		public override void Execute(IBusinessOperation businessOperation)
		{
			var bo = (RuntimeBusinessOperation)businessOperation;
			var order = bo.Order;

			var totValue = ExpressionCalculator.GetValue(bo.Expression, bo.Order);

			order.Total = decimal.Parse(totValue.ToString()); 
			bo.Total = decimal.Parse(totValue.ToString());
		}
	}
}
