﻿using System.Text;
using DevExpress.ExpressApp;
using Ticket36921.Module.BusinessOperations;
using Ticket36921.Module.Settings;
using Xafari.BC;
using Xafari.BC.BusinessOperations;
using Xafari.BC.BusinessOperations.Controllers;
using Xafari.BC.Settings;
using Xafari.Utils;

namespace Ticket36921.Module.Controllers
{
	// For more typical usage scenarios, be sure to check out https://documentation.devexpress.com/eXpressAppFramework/clsDevExpressExpressAppViewControllertopic.aspx.
	public partial class CustomizeBusinessOperationsController : ViewController
	{
		private BOExecViewController _boExecViewController;

		public CustomizeBusinessOperationsController()
		{
			InitializeComponent();
			// Target required Views (via the TargetXXX properties) and create their Actions.
		}

		protected override void OnActivated()
		{
			base.OnActivated();
			_boExecViewController = Frame.GetController<BOExecViewController>();
			if (_boExecViewController != null)
			{
				_boExecViewController.BOEvents[typeof(RuntimeBusinessOperation)].ProcessResult += CalcOrderTotalProcessResult;
				_boExecViewController.BOEvents[typeof(RuntimeBusinessOperation)].ContextPropertyInitializing += CustomizeBusinessOperationsController_ContextPropertyInitializing;
			}
		}

		private void CustomizeBusinessOperationsController_ContextPropertyInitializing(object sender, ContextPropertyInitializingEventArgs e)
		{
			var bo = e.BusinessOperation;

			this.ObjectSpace.AppSettings().Update();
			var expressionValue = this.ObjectSpace.AppSettings().Root().KeysProjectSettings().BusinessOperations
				.RuntimeBusinessOperation.Expression.GetValue();

			if (expressionValue!= null && bo is RuntimeBusinessOperation runtime)
				runtime.Expression = expressionValue.ToString();
		}

		private void CalcOrderTotalProcessResult(object sender, ProcessResultEventArgs e)
		{
			var businessOperation = (RuntimeBusinessOperation)e.BusinessOperation;

			var resultMsg = new StringBuilder();
			resultMsg.AppendLine(string.Format("Orser Number : {0}", businessOperation.OrderNum));
			resultMsg.AppendLine(string.Format("Total : {0}", businessOperation.Total));

			MessageObject.CreateMessageInfo(
				e.ShowViewParameters,
				Application,
				"Result",
				resultMsg.ToString()
			);
		}

		protected override void OnViewControlsCreated()
		{
			base.OnViewControlsCreated();
			// Access and customize the target View control.
		}
		protected override void OnDeactivated()
		{
			if (_boExecViewController != null)
			{
				_boExecViewController.BOEvents[typeof(RuntimeBusinessOperation)].ProcessResult -= CalcOrderTotalProcessResult;
			}

			// Unsubscribe from previously subscribed events and release other references and resources.
				base.OnDeactivated();
		}
	}
}
