﻿using System;
using System.Linq;
using DevExpress.ExpressApp;
using DevExpress.Data.Filtering;
using DevExpress.Persistent.Base;
using DevExpress.ExpressApp.Updating;
using DevExpress.Xpo;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Persistent.BaseImpl;
using Ticket36921.Module.BusinessObjects;
using Ticket36921.Module.Settings;
using Xafari.BC;
using Xafari.BC.Services;
using Xafari.BC.Settings;

namespace Ticket36921.Module.DatabaseUpdate
{
	public class Updater : XafariModuleUpdater
	{
		public Updater(IObjectSpace objectSpace, Version currentDBVersion) :
			base(objectSpace, currentDBVersion)
		{
		}

		public override void SafeUpdateDatabaseAfterUpdateSchema()
		{
			base.SafeUpdateDatabaseAfterUpdateSchema();
			CreateCustomer();
			CreateObjects();
			SettingsAccessor.Instance.Activated += Instance_Activated;
			//string name = "MyName";
			//DomainObject1 theObject = ObjectSpace.FindObject<DomainObject1>(CriteriaOperator.Parse("Name=?", name));
			//if(theObject == null) {
			//    theObject = ObjectSpace.CreateObject<DomainObject1>();
			//    theObject.Name = name;
			//}
		}

		private void Instance_Activated(object sender, EventArgs e)
		{
			var objectSpace = ObjectSpaceFactory.Instance.GetObjectSpace();
			var expression = objectSpace.AppSettings().Root().KeysProjectSettings().BusinessOperations
				.RuntimeBusinessOperation
				.Expression.Value;
			if (expression != null) return;
			objectSpace.AppSettings().Update();
			objectSpace.AppSettings().Root().KeysProjectSettings().BusinessOperations.RuntimeBusinessOperation
				.Expression.Value = "[OrderDetails.UnitPrice]*[OrderDetails.Quantity]";
			objectSpace.CommitChanges();
		}

		public override void SafeUpdateDatabaseAfterLogon()
		{
			base.SafeUpdateDatabaseAfterLogon();
			// add updater code after logon action completed
		}

		private void CreateCustomer()
		{
			if (this.ObjectSpace.GetObjects<Customer>().Count != 0) return;

			var customer = this.ObjectSpace.CreateObject<Customer>();

			customer.Name = "Name_1";
			customer.Discount = 5;

			this.ObjectSpace.CommitChanges();
		}

		private void CreateObjects()
		{
			if (this.ObjectSpace.GetObjects<Order>().Count != 0) return;

			var order = this.ObjectSpace.CreateObject<Order>();

			order.Number = "001";
			order.Customer = this.ObjectSpace.GetObjects<Customer>().First();
			order.Freight = Decimal.One;

			var orderDetails = this.ObjectSpace.CreateObject<OrderDetails>();

			orderDetails.Order = order;
			orderDetails.Discount = 2;
			orderDetails.Quantity = 3;
			orderDetails.UnitPrice = 10;

			this.ObjectSpace.CommitChanges();

		}
	}
}
